\ Output morse code to sound device
\ Source text for DX-Forth
\ Pavel Zampach, 2020
\ *

include sound                           \ enable word SOUND
decimal
100 value DOT                           \ length of dot [ms]
300 value DASH                          \ length of dash [ms]
700 value SPCPAU                        \ length of space [ms]
4000 constant PITCH                     \ frequency of sound [Hz]

\ timing, length of dot [ms]
: MTIMING     ( n -- )
  to dot
  dot 3 * to dash
  dot 7 * to spcpau
;

\ bits 0-2  = length of code
\ bits 7-3  = set: dash, reset: dot
create CODETAB
$42 c, $84 c, $A4 c, $83 c,  \ ABCD
$01 c, $24 c, $C3 c, $04 c,  \ EFGH
$02 c, $74 c, $A3 c, $44 c,  \ IJKL
$C2 c, $82 c, $E3 c, $64 c,  \ MNOP
$D4 c, $43 c, $03 c, $81 c,  \ QRST
$23 c, $14 c, $63 c, $94 c,  \ UVWX
$B4 c, $C4 c, $FD c, $7D c,  \ YZ01
$3D c, $1D c, $0D c, $05 c,  \ 2345
$85 c, $C5 c, $D5 c, $F5 c,  \ 6789

\ output char by index in codetab
: MCODE       ( idx -- )
  codetab + c@                          \ read code from table
  dup 7 and 0 do
    dup $80 and if dash else dot then   \ set duration
    pitch swap sound   dot ms           \ sound and pause after
    1 lshift                            \ next
  loop   drop
  dash ms                               \ pause after char
;

\ output char (only letters, digits and space are allowed)
: MEMIT       ( char -- )
  dup bl = if drop spcpau ms end        \ space
  dup [char] 0 - 10 u< if               \ digits
    26 + mcode
  end
  upcase [char] A - dup 26 u< if        \ letters
    mcode
  end
  drop
;

\ morse equivalent of TYPE
: MTYPE       ( adr u -- )
  0 ?do
    count memit
  loop   drop
;
